package com.monead.demo;

/**
 * This application is for demonstration purposes only.
 * It must not be used in a production environment. 
 * 
 * The algorithm for calculating the response is
 * not robust and provides no security. 
 */

import java.text.DecimalFormat;
import java.text.NumberFormat;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.SystemClock;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

/**
 * An Android "Activity" is a class meant to be used
 * for creating interactive applications.
 * 
 * This activity was created as a demonstration of 
 * the basic building blocks of an Android
 * application for a presentation at the 
 * Capital District Mobile Developers Network
 * on February 1, 2012.
 * 
 * @author David Read
 *
 */
public class Tvmdn extends Activity {
  /** 
   * Called when the activity is first created. 
   */
  @Override
  public void onCreate(Bundle savedInstanceState) {
    super.onCreate(savedInstanceState);
    setContentView(R.layout.main);

    setup();
  }

  /** 
   * Called when activity loses foreground control. 
   */
  public void onPause() {
    super.onPause();

    clearValues();
  }

  /**
   * Called when activity regains foreground control.
   */
  public void onResume() {
    super.onResume();

    setForInput();
  }

  /**
   * Set focus and bring up soft keyboard.
   */
  private void setForInput() {
    (new Handler()).postDelayed(new Runnable() {
      public void run() {
        EditText pin = (EditText) findViewById(R.id.pin);
        
        pin.requestFocus();
        
        pin.dispatchTouchEvent(MotionEvent.obtain(
            SystemClock.uptimeMillis(), SystemClock.uptimeMillis(),
            MotionEvent.ACTION_DOWN, 0, 0, 0));
        pin.dispatchTouchEvent(MotionEvent.obtain(
            SystemClock.uptimeMillis(), SystemClock.uptimeMillis(),
            MotionEvent.ACTION_UP, 0, 0, 0));

      }
    }, 150);
  }

  /**
   * Clear all the inputs and outputs.
   */
  private void clearValues() {
    EditText pin;
    EditText challenge;
    TextView response;

    pin = (EditText) findViewById(R.id.pin);
    challenge = (EditText) findViewById(R.id.challenge);
    response = (TextView) findViewById(R.id.response);

    pin.setText("");
    challenge.setText("");
    response.setText("");
  }

  /**
   * Setup the application - register a listener
   * for the button.
   */
  private void setup() {
    Button calculate = (Button) findViewById(R.id.button_execute);
    calculate.setOnClickListener(new OnClickListener() {
      public void onClick(View v) {
        calculateResponse();
      }
    });
  }

  /**
   * Calculate the security response.
   * 
   * This is a trivial implementation and is not
   * suitable for an actual authentication
   * mechanism.
   */
  private void calculateResponse() {
    EditText pin;
    EditText challenge;
    TextView response;
    String androidId;
    int pinValue;
    int challengeValue;
    int androidIdValue;
    int remainder;

    pin = (EditText) findViewById(R.id.pin);
    challenge = (EditText) findViewById(R.id.challenge);
    response = (TextView) findViewById(R.id.response);

    androidId = ""
        + android.provider.Settings.Secure.getString(getContentResolver(),
            android.provider.Settings.Secure.ANDROID_ID);

    try {
      androidIdValue = calcAndroidIdValue(androidId);
      pinValue = Integer.parseInt(pin.getText().toString().trim());
      challengeValue = Integer.parseInt(challenge.getText().toString().trim());

      if (pinValue > 10000 || pinValue < 1000) {
        throw new IllegalStateException("Pin must be a value from 1000 to 9999");
      }
      if (challengeValue > 10000 || challengeValue < 1000) {
        throw new IllegalStateException(
            "Challenge must be a value from 1000 to 9999");
      }

      remainder = ((pinValue * challengeValue) / (androidIdValue / 31)) % 10000;
      NumberFormat format;
      format = new DecimalFormat("0000");

      response.setText(format.format(remainder));
    }
    catch (Throwable throwable) {
      Log.e("ChallengeResponse", "Failed to calculate response", throwable);
      response.setText(getResources().getString(R.string.default_response));
    }
  }

  /**
   * Get a 4-digit value based on the device's id.
   * 
   * @param androidId
   *          device's id
   * @return 4-digit value based on id
   */
  private int calcAndroidIdValue(String androidId) {
    int[] positionValue;
    int totalValue;
    int factor;

    positionValue = new int[4];
    totalValue = 0;

    for (int position = 0; position < positionValue.length; ++position) {
      positionValue[position] = 0;
    }

    for (int set = 0; set < 4; ++set) {
      for (int digit = 0; digit < 4; ++digit) {
        Log.d("ChallengeResponse", "calcAndroidId[" + set + "," + digit + "]="
            + androidId.substring((set * 4) + digit, (set * 4) + digit + 1));
        positionValue[digit] += Integer.parseInt(
            androidId.substring((set * 4) + digit, (set * 4) + digit + 1), 16);
      }
    }

    factor = 1000;
    for (int position = 0; position < positionValue.length; ++position) {
      positionValue[position] %= 10;
      totalValue += (factor * positionValue[position]);
      factor /= 10;
    }

    return totalValue;
  }
}
